from tqdm.auto import tqdm

from ..utils import logger, constants
from ..services import *

log = logger.setup_logger()


def upload_model(
    folder: str,
    repo_id: str,
    model_name: str,
):
    """
    upload a local dir (list of model files) to OpenI, not subfolder allowed

    :param folder: local folder name
    :param repo_id: repo id in form of `owner/repo_name`
    :param model_name: name of model shown on web page
    :return:

    e.g.
    >>> from openi.model import upload_model
    >>> upload_model("local_dir/chatglm2", "openi/official", "chatglm2b")

    Calculating file md5… (13/13) |███████████████████████████████████████████████████████████████████████████████| [00:00]
    Complete(…kenizer_config.json): 100%|█████████████████████████████████████████████████| 244B/244B [00:00<00:00, 795B/s]
    Complete(         config.json): 100%|███████████████████████████████████████████| 1.19kB/1.19kB [00:00<00:00, 1.71kB/s]
    Complete(      .gitattributes): 100%|███████████████████████████████████████████| 1.48kB/1.48kB [00:00<00:00, 1.15kB/s]
    Complete(…guration_chatglm.py): 100%|█████████████████████████████████████████████| 2.19kB/2.19kB [00:01<00:00, 859B/s]
    Complete(       MODEL_LICENSE): 100%|█████████████████████████████████████████████| 4.04kB/4.04kB [00:01<00:00, 686B/s]
    Complete(           README.md): 100%|█████████████████████████████████████████████| 7.89kB/7.89kB [00:01<00:00, 575B/s]
    Complete(…nization_chatglm.py): 100%|█████████████████████████████████████████████| 9.83kB/9.83kB [00:02<00:00, 491B/s]
    Complete(     quantization.py): 100%|█████████████████████████████████████████████| 14.3kB/14.3kB [00:02<00:00, 432B/s]
    Complete(…odel.bin.index.json): 100%|█████████████████████████████████████████████| 20.0kB/20.0kB [00:02<00:00, 384B/s]
    Complete( modeling_chatglm.py): 100%|█████████████████████████████████████████████| 49.5kB/49.5kB [00:02<00:00, 344B/s]
    Complete(     tokenizer.model): 100%|███████████████████████████████████████████████| 995kB/995kB [00:03<00:00, 311B/s]
    Uploading(…-00001-of-00007.bin):   35%|██████████████████████                           | 0.34GB/0.98GB [00:03<?, ?B/s]
    Waiting(…-00002-of-00007.bin):   0%|                                                     | 0.00B/1.69GB [00:00<?, ?B/s]

    """

    # OpeniFileUpload
    local_dir, targe_files = get_upload_folder_files(folder)
    if targe_files == []:
        msg = f"❌ {local_dir} 未找到任何本地文件"
        log.error(msg)
        raise ValueError(msg)

    # OpeniRepo, check repo_id, access
    openi_repo = OpeniRepo(repo_id=repo_id)

    # OpeniModel
    openi_model = OpeniModel(
        model_name=model_name,
        repo_id=openi_repo.repo_id,
    )
    if openi_model.can_operate is False:
        msg = f"❌ `{openi_repo.current_user}` 无此模型上传权限，模型名称 `{openi_model.model_name}`, 仓库名称 `{openi_model.repo_id}`"
        log.error(msg)
        raise ValueError(msg)
    if openi_model.model_type == 0:
        msg = f"❌ `{openi_model.model_name}` 为线上模型，无法上传本地文件"
        log.error(msg)
        raise ValueError(msg)

    # openifiles
    tiltle_pbar = tqdm(
        total=len(targe_files),
        leave=True,
        bar_format="{desc}({n_fmt}/{total_fmt}) |{bar}| [{elapsed}]",
        desc=f"Calculating file md5… ",
        position=0,
    )
    openifiles = list()
    for file in targe_files:
        openi_file = OpeniFileUpload(
            local_file=file,
            repo_id=openi_repo.repo_id,
            upload_type=constants.FILE.DEFAULT_UPLOAD_TYPE,
            dataset_or_model_id=openi_model.model_id,
            dataset_or_model_name=openi_model.model_name,
            target_type="model",
        )
        tiltle_pbar.update(1)
        openifiles.append(openi_file)
    # tiltle_pbar.close()

    # # Display pbar
    display_progress_bar(openifiles)

    # # Upload
    for openi_file in openifiles:
        openi_file.upload_with_tqdm()



import logging
import os
import json
import time
from datetime import datetime

from .constants import *


def setup_logger():
    LOG_FORMAT = "%(asctime)s [%(levelname)s] %(funcName)s(): %(message)s"  # %(filename)s %(funcName)s():
    DATE_FORMAT = "%Y/%m/%d %H:%M:%S"
    LOG_SUFFIX = time.strftime("%Y%m%d")
    LOG_FILE = os.path.join(PATH.LOG_PATH, f"{LOG_SUFFIX}.log")
    logging.addLevelName(25, "HTTP")
    logging.addLevelName(26, "RESPONSE")

    logging.basicConfig(
        filename=LOG_FILE,
        level=logging.INFO,
        format=LOG_FORMAT,
        datefmt=DATE_FORMAT,
    )
    logger = logging.getLogger(__name__)

    def http(self, message, *args, **kws):
        if self.isEnabledFor(25):
            self._log(25, message, args, **kws)

    def response(self, message, *args, **kws):
        if self.isEnabledFor(26):
            self._log(26, message, args, **kws)

    logging.Logger.http = http
    logging.Logger.response = response

    return logger


def get_time():
    return datetime.now().strftime("%H:%M:%S")


def algnprint(input_dict):
    max_key_length = max(len(key) for key in input_dict.keys()) + 1
    for key, value in input_dict.items():
        key += ":"
        formatted_key = f"{key:{max_key_length}}"
        print(f"{formatted_key} {value}")


def jprint(dict):
    print(json.dumps(dict, indent=4, ensure_ascii=False))


def jlog(dict):
    return json.dumps(dict, indent=4, ensure_ascii=False)


def jsave(dict, dir=None):
    with open(dir, "w+") as f:
        json.dump(dict, f, indent=4, ensure_ascii=False)
    print(f"jsave to {dir}")


def lprint(list, with_index=True):
    max_index_width = len(str(len(list) - 1))
    for index, item in enumerate(list):
        if with_index:
            index_str = str(index).rjust(max_index_width)
            print(f"{index_str} {item}")
        else:
            print(item)



import os
import re

from ..utils import logger, constants
from ..services import *

log = logger.setup_logger()


def download_file(
    file: str,
    repo_id: str,
    cluster: str = "npu",
    save_path: str = constants.PATH.DATASET_SAVE_PATH,
):
    """
    download a zip or tar.gz dataset file from OpenI

    :param file: dataset file name
    :param repo_id: repo id in form of `owner/repo_name`
    :param cluster: cluster type in [gpu,npu], default is `npu`
    :param save_path: local save path, default is `./dataset`
    :return:

    e.g.
    >>> from openi.dataset import download_file
    >>> download_file("cifar10.zip", "openi/official", cluster="gpu", save_path="mydata")

    Complete( cifar10.zip)(gpu): 100%|████████████████████████| 163MB/163MB [00:00<00:00, 54.0MB/s]

    """

    upload_type = get_upload_type(cluster)

    # Define a regex pattern to match the desired formats
    pattern = r"^\w+\.(zip|tar\.gz)$"
    if not re.match(pattern, file):
        msg = f"`{file}` 非压缩文件（zip，tar.gz）无法通过sdk下载"
        log.error(msg)
        raise ValueError(msg)

    openi_repo = OpeniRepo(repo_id=repo_id)
    if openi_repo.access not in ["read", "write"]:
        msg = f"`{repo_id}` 无权限操作此仓库"
        log.error(msg)
        raise ValueError(msg)

    openi_dataset = OpeniDataset(repo_id=openi_repo.repo_id)
    openi_dataset.get_dataset_file(upload_type=upload_type)
    targe_file = None
    log.info(f"openi_dataset.dataset_files,{openi_dataset.dataset_files}")
    if openi_dataset.dataset_files is not None:
        for f in openi_dataset.dataset_files:
            if f["name"] == file:
                targe_file = f
    log.info(f"targe_file,{targe_file}")
    if openi_dataset.dataset_files is None or targe_file is None:
        msg = (
            f"❌ `{openi_repo.repo_id}`-`{openi_dataset.dataset_name}` "
            f"数据集内未找到名为{file}({cluster})的文件"
        )
        log.error(msg)
        raise ValueError(msg)

    local_dir = os.path.expanduser(save_path)
    if not os.path.exists(local_dir):
        os.makedirs(local_dir, exist_ok=True)

    openi_file = OpeniFileDownload(
        filename=targe_file["name"],
        size=targe_file["size"],
        local_dir=local_dir,
        repo_id=openi_repo.repo_id,
        dataset_or_model_id=openi_dataset.dataset_id,
        dataset_or_model_name=openi_dataset.dataset_name,
        target_type="dataset",
        upload_type=upload_type,
        uuid=targe_file["uuid"],
    )

    display_progress_bar([openi_file])
    openi_file.download_with_tqdm()
